% Preliminary checks, fit or not fit, calculate model curves, and plot
% everything. Normally, there will be no need to change this part of the
% code. However, if you want to change the plotting, this is the place.
% 
% Author : Tjalling Jager 
% Date:   Aug 2015
% Web support: http://www.debtox.info/byom.html

prelim_checks % call the script that does some preliminary checks on the data

if ~exist('Xlo','var') % argument is used when called by calc_conf
    tic % turn on the stopwatch
    % this is needed, otherwise, the stopwatch will be reset when this
    % script is called for a second time by calc_conf.m.
end

%% Fit model parameters 

zvd_plot = 1; % turn on the plotting of zero-variate data (if defined)

bw    = glo.plt(1); % if set to 1, plots in black and white with different plot symbols
cn    = glo.plt(2); % if set to 1, connect model line to points (only for bw=1)
limax = glo.plt(3); % if set to 1, limit axes to the data set for each stage

if fit == 1 && ~exist('Xlo','var') % last argument is used when called by calc_conf
    if ~exist('it','var')
        it = 1; % show iterations unless specified not to
    end
    [par_out,FVAL] = start_calc(par,mfilename_base,it); % start the optimisation
    % optimised parameter values are displayed by this function and also
    % returned in the structure par_out
else
    par_out = par; % don't fit but only simulate
end

%% Calculate model curves for plotting

% initialise the matrices that will collect the output (time x scenario)
for i = 1:n_X % run through state variables
    Xall{i} = zeros(length(t),n_s); % initialise state variable output with zeros
end
% tt = zeros(1,n_s);         % event times
for i = 1:n_X2 % run through additional data sets
    Xall2x{i} = zeros(length(t),n_s); % initialise additional x-data output with zeros
    Xall2y{i} = zeros(length(t),n_s); % initialise additional y-data output with zeros
end

for j = 1:n_s % run through our scenarios
    
    if n_X2 > 0 % if there are additional univariate data ... 
        [Xout,TE,Xout2] = call_deri(t,par_out,X0mat(:,j)); % use call_deri.m to provide the output for one scenario
    else % this is needed to ensure backwards compatability
        [Xout,TE] = call_deri(t,par_out,X0mat(:,j)); % use call_deri.m to provide the output for one scenario
    end
    
    for i = 1:n_X  % run through state variables
        Xall{i}(:,j) = Xout(:,i); % collect state variable i into structure X
    end
    tt{j} = TE; % remember where the event took place
    
    for i = 1:n_X2, % now loop over the additional data sets
        Xall2x{i}(:,j) = Xout2{i}(:,1); % collect the new x-values
        Xall2y{i}(:,j) = Xout2{i}(:,2); % collect the new y-values
    end
    
    % this creates a cell array for the figure legends from the first
    % row of X0mat
    L{j} = [leglab1,num2str(X0mat(1,j)),leglab2];
end

if n_D > 1
    % this is to ensure that when there are more data series, that we know
    % which scenarios are in which data series
    for k = 1:n_D % run through number of data sets per state
        c_all = [];
        for i = 1:n_X % run through state variables
            c_all = [c_all DATA{k,i}(1,2:end)];
        end
        ck{k} = unique(c_all);
    end
end

for k = 1:n_D % run through number of data sets per state
    for i = 1:n_X % run through state variables
        if sho == 1 % show all scenario's in X0mat
            if n_D > 1
                % when there are more data series, plot only model curves
                % relevant for THAT series
                [dummy,loci]=ismember(ck{k},X0mat(1,:)); % loci is location of state i in the scenario range
                if dummy ~= 0
                    loc{k,i} = loci;
                else
                    loc{k,i} = [];
                end
            else
                loc{k,i} = 1:size(Xall{i},2);
            end
            
        else % otherwise, only show model results for treatments were there are data
            [dummy,loci]=ismember(DATA{k,i}(1,2:end),X0mat(1,:)); % loci is location of state i in the scenario range
            loci(loci==0) = []; % remove zeros (where data for certain scenarios are ignored)
            loci = unique(loci); % only keep the unique entries (in case of replicated data)
            % X{i} = X{i}(:,loci); % use loci to extract only the model values for scenarios used in data set i
            loc{k,i} = loci; % put the result in a structure for later use
        end
        if ~isempty(loc{k,i})
            Le{k,i} = L(loc{k,i}); % use loc{i} to extract only legend entries for scenarios used in data set i
        else
            Le{k,i} = [];
        end
    end
end

%% Plot model curves with data points

plotcol = 'krbgcm'; % 6 colors should be enough ...
markers_dr = {'s','^','o','d','v','p','>','<'}; % complete set of 8 markers

% make one plot with subplots for each state variable
n = ceil(sqrt(n_X));
m = ceil(n_X/n);

locall = [];
for k = 1:n_D % run through number of data sets per state
    for i = 1:n_X % run through state variables
        locall = [locall loc{k,i}];
    end
end
locall = unique(locall);

for k = 1:n_D % run through number of data sets per state
    figure % new figure for each set of data ...
    
    for i = 1:n_X % run through state variables
        plotpts = [];
        h  = []; % clear handle for plotting for plot symbols (BW)
        hc = []; % clear handle for plotting connection lines (BW)
        max_data = 0; % clear maximum for data (to limit axis bounds if needed)
        min_data = +inf; % clear minimum for data (to limit axis bounds if needed)
        
        % make a subplot for state i
        subplot(m,n,i)
        hold on
        
        % add a title to the graph with the filename in it
        if n_D == 1
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter','none','FontSize',10)
        else
            title(['Plotted from: ',mfilename_base, ' (',date,'), data series ',num2str(k),' of ',num2str(n_D)],'Interpreter','none','FontSize',10)
        end
        % add labels to the axes
        xlabel(xlab,'FontSize',12)
        ylabel(ylab{i},'FontSize',12)
        set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
        
        loci = loc{k,i};
        for j = 1:length(loci) % run through all scenarios that have data for state 1
            [dummy,locj] = ismember(loci(j),locall); % to find the right colour ...
            if bw == 1 % plot in black and white
                plot(t,Xall{i}(:,loci(j)),'k-','LineWidth',1) % plot each model curve with a black line
            else
                if locj <= 6
                    plot(t,Xall{i}(:,loci(j)),[plotcol(locj),'-'],'LineWidth',2) % plot each model curve with a different color
                elseif locj <= 12 % if we have more than 6 treatments, re-use colours and make broken line
                    plot(t,Xall{i}(:,loci(j)),[plotcol(locj-6),'--'],'LineWidth',2) % plot each model curve with a different color
                elseif locj <= 18 % if we have more than 12 treatments, re-use colours and make dot-stripe line
                    plot(t,Xall{i}(:,loci(j)),[plotcol(locj-12),'.-'],'LineWidth',2) % plot each model curve with a different color
                else
                    disp('cannot plot more than 18 lines yet ... so modify your script')
                end
            end
            % Note: if sho=0 this only plots scenarios that actually have data!
        end
        
        if bw ~= 1 % plot in color: then use colors for the legend
            % add a legend using the cell array L
            if ~isempty(Le{k,i})
                legend(Le{k,i}) % create a legend
            end
        end
        
        plotdata = DATA{k,i};
        if plotdata(1,1) == -1 % if we have survival data ...
            a = plotdata(2:end,2:end); % extract data set
            w = W{k,i}; % extract weight factors
            % code below is from my DEBtoxM code in plot_all. This is a bit
            % awkward, but is needed when there are animals missing or
            % removed during the experiment. What is plotted is the
            % corrected data!
            if sum(w(:)) == 0, % if there are NO missing data ...
                a = a ./ (ones(size(a,1),1)*a(1,:)); % turn them into survival probabilities
            else % if there are missing data ...
                if ~any(isnan(a(:))) % does the dataset NOT contain nans?
                    divideby = (a(1:end-1,:)-w(1:end-1,:));
                    divideby(divideby==0) = 1e-6; % to avoid division by zero!
                    a = cumprod([ones(1,size(a,2)); a(2:end,:)./divideby]);
                else % it does contain NaNs, which makes life miserable
                    w2 = w;
                    w2(isnan(a)) = NaN; % also make corresponding points in w2 a NaN
                    indnan = cumsum(sum(isfinite(w2),1)); % linear indices to last datapoint for each conc, excl NaNs
                    indnan(end) = []; % remove last one (does not overlap with next conc)
                    w2 = w2(:); % make them into a column vector
                    Z2 = a(:);
                    w2(isnan(w2)) = []; % and remove the NaNs
                    Z2(isnan(Z2)) = [];
                    divideby = (Z2(1:end-1)-w2(1:end-1));
                    divideby(divideby==0) = 1e-6; % to avoid division by zero!
                    Z3 = Z2(2:end)./divideby; % corrected survival fraction
                    Z3(indnan) = []; % remove the overlapping data with next concentration
                    Z4 = nan(size(a)); % prepare a new matrix with NaNs
                    Z4(1,:) = []; % but take out the first row (will become ones)
                    Z4(isfinite(a(2:end,:))) = Z3; % put the corrected fraction back in a matrix at the correct positions
                    Z4 = [ones(1,size(a,2));Z4]; % and add a row with ones on top
                    Z4(isnan(a))=1; % replace NaNs by one to be able to do cumprod
                    Z4 = cumprod(Z4); % cumulative product to get overall survival fraction
                    Z4(isnan(a)) = NaN; % put the NaNs back in their position
                    a = Z4; % this is the corrected data to plot!
                end
            end
            plotdata(2:end,2:end) = a; % and put them back
            ylim([0 1.02]); % restrict axis from 0-1
        end
        
        td = plotdata(2:end,1); % time vector for this data set
        for j = 1:length(loci) % run through all scenarios that have data for state 1
            ind_plot = plotdata(1,:) == X0mat(1,loci(j)); % location to data to plot
            ind_plot(1) = 0; % zero makes sure that time column is not plotted itself

            if sum(ind_plot) > 0 % this is needed in case we simulate and don't fit
                plotpts = plotdata(2:end,ind_plot);

                if repls == 0 % plot means instead of replicates
                    plotmn = zeros(length(td),1); % initialise mean plot vector with zeros
                    for jk = 1:length(td)
                        plotmn(jk,1) = mean(plotpts(jk,~isnan(plotpts(jk,:))));
                        % note that this is the normal mean, even if the
                        % data are transformed in some way!
                    end
                    plotpts = plotmn;
                end
                
                max_data = max(max_data,max(plotpts(:))); % remember maximum of plotted data
                min_data = min(min_data,min(plotpts(:))); % remember minimum of plotted data
                
                [dummy,locj] = ismember(loci(j),locall); % to find the right colour ...
                if bw == 1 % plot in black and white
                    if cn == 1; % plot connection between line and point
                        plotcon = interp1(t,Xall{i}(:,loci(j)),td); % model prediction at measured data points
                        for con = 1:length(td)
                            for con2 = 1:size(plotpts,2)
                                if ~isnan(plotpts(con,con2)) % only do that when there is a data point
                                    hc{j}(con,con2) = plot([td(con) td(con)],[plotcon(con) plotpts(con,con2)],'k:','LineWidth',1);
                                end
                            end
                        end
                        plotcon_tmp(:,loci(j)) = plotcon; % remember the predictions at each data point
                        % % I commented this out; it was used for the analysis of simulated data with GUTS
                    end
                    
                    if locj <= 8
                        htmp = plot(td,plotpts,['k',markers_dr{locj}],'MarkerFaceColor','w');
                        h(j) = htmp(1); % needed for replicates
                    elseif locj <= 16
                        htmp = plot(td,plotpts,['k',markers_dr{locj-8}],'MarkerFaceColor','k');
                        h(j) = htmp(1); % needed for replicates
                    elseif locj <= 24
                        htmp = plot(td,plotpts,['k',markers_dr{locj-16}],'MarkerFaceColor','y');
                        h(j) = htmp(1); % needed for replicates
                    end
                else
                    if locj <= 6
                        plot(td,plotpts,'ko','MarkerFaceColor',plotcol(locj))
                    elseif locj <= 12 % if we have more than 6 treatments, re-use colours and make squares
                        plot(td,plotpts,'ks','MarkerFaceColor',plotcol(locj-6))
                    elseif locj <= 18 % if we have more than 12 treatments, re-use colours and make triangles
                        plot(td,plotpts,'k^','MarkerFaceColor',plotcol(locj-12))
                    end
                end
            end
        end
        
        if limax == 1 && ~isempty(plotpts)
            axis([min(td) max(td)*1.02 min_data/1.02 max_data*1.02]); % limit axes to data
        end
        
        if exist('Xlo','var') % do we have max and min model lines from the slice sampling?
           
            switch type_conf
                case 1
                    title('Intervals: 95% of model predictions from sample of posterior')
                case 2
                    title('Intervals: min-max of model predictions from 95% likelihood-based region')
                case 3
                    title('Intervals: min-max of model predictions from 95% highest-posterior density region')
            end
 
            for j = 1:length(loci) % run through all scenarios that have data for state 1
                [dummy,locj] = ismember(loci(j),locall); % to find the right colour ...
                if locj <= 6
                    plot(t,Xlo{i}(:,loci(j)),[plotcol(locj),':'],'LineWidth',2) % plot each model curve with a different color
                    plot(t,Xhi{i}(:,loci(j)),[plotcol(locj),':'],'LineWidth',2) % plot each model curve with a different color
                elseif locj <= 12 % if we have more than 6 treatments, re-use colours
                    plot(t,Xlo{i}(:,loci(j)),[plotcol(locj-6),':'],'LineWidth',2) % plot each model curve with a different color
                    plot(t,Xhi{i}(:,loci(j)),[plotcol(locj-6),':'],'LineWidth',2) % plot each model curve with a different color
                elseif locj <= 18 % if we have more than 12 treatments, re-use colours
                    plot(t,Xlo{i}(:,loci(j)),[plotcol(locj-12),':'],'LineWidth',2) % plot each model curve with a different color
                    plot(t,Xhi{i}(:,loci(j)),[plotcol(locj-12),':'],'LineWidth',2) % plot each model curve with a different color    
                end
            end
        end
       
        if bw == 1 % plot in black and white: then use symbols for the legend
            % add a legend using the cell array L
            if ~isempty(Le{k,i})
                legend(h(h>0),Le{k,i}(find(h>0))) % create a legend
            end
            if cn == 1 % move the conncention line to the background of the plot
                for j=1:length(hc) % run through all scenarios that have data for state 1
                    for j2 = 1:size(hc{j},2)
                        h_tmp = hc{j}(:,j2);
                        h_tmp = h_tmp(h_tmp~=0); % remove zeros for NaNs that were not connected
                        if isreal(h_tmp) % in older versions, the handles are numbers
                            uistack(h_tmp,'bottom') % which makes it easy to send them to the back
                        else % they are something nasty :-(
                            for j3 = 1:length(h_tmp)
                                if length(class(h_tmp(j3))) ~=35 % sometimes it is a placeholder, and we cannot use that
                                    uistack(h_tmp(j3),'bottom')
                                end
                            end
                        end
                    end
                end
            end
        end
        
    end
end

%% Plot zero-variate data points

if fit == 1 && ~exist('Xlo','var') % only when fitted and not called by calc_conf
    if zvd_plot == 1 % Plot zero-variate data in a nice way ...
        global namesz
        if ~isempty(zvd) && ~isempty(namesz)
            figure
            for i = 1:length(namesz) % go through contents of the global zvd
                a = eval(['zvd.',namesz{i}]); % extract zero-variate info
                errorbar(i-0.1,a(1),1.96*a(2),1.96*a(2),'ko','MarkerFaceColor','w')
                hold on
                plot(i+0.1,a(3),'ko','MarkerFaceColor','y')
                if i == 1
                    legend('zero-var. data with CI','model result')
                end
            end
            xlim([0 i+1]);
            ym = ylim; % ask for current y limits
            ylim([0 ym(2)*1.02]); % start at zero, but keep upper limit
            set(gca,'xtick',1:i,'xticklabel',namesz,'LineWidth',1,'FontSize',12)
            xlabel('zero-variate data point name','FontSize',12)
            ylabel('zero-variate value','FontSize',12)
        end
    end
end

%% Plot extra data points on different x-axes

if n_X2 > 0 % if there are additional data fitted ...
    plotcol = 'krbgcm'; % 6 colors should be enough ...
    markers_dr = {'s','^','o','d','v','p','>','<'}; % complete set of 8 markers

    % make one plot with subplots for each data set
    n = ceil(sqrt(n_X2));
    m = ceil(n_X2/n);
    
    figure

    for i = 1:n_X2 % run through extra data sets
        h = [];
        % make a subplot for set i
        subplot(m,n,i)
        hold on
        
        % add a title to the graph with the filename in it
        title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter','none','FontSize',10)
        % add labels to the axes
        xlabel(xlab2{i},'FontSize',12)
        ylabel(ylab2{i},'FontSize',12)
        set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting

        % plot model curves only for scenarios that have extra data
        % loci = DATAx{i}(1,2:end); % scenarios in this extra data set
        [dummy,loci]=ismember(DATAx{i}(1,2:end),X0mat(1,:)); % loci is location of state i in the scenario range
        loci(loci==0) = []; % remove zeros (where data for certain scenarios are ignored)
        loci = unique(loci); % only keep the unique entries (in case of replicated data)
        
        for j = 1:length(loci) % run through all scenarios that have data for state 1
            [dummy,locj] = ismember(loci(j),locall); % to find the right colour ...
            if bw == 1 % plot in black and white
                plot(Xall2x{i}(:,loci(j)),Xall2y{i}(:,loci(j)),'k-','LineWidth',1) % plot each model curve with a black line
            else
                if locj <= 6
                    h(j) = plot(Xall2x{i}(:,loci(j)),Xall2y{i}(:,loci(j)),[plotcol(locj),'-'],'LineWidth',2); % plot each model curve with a different color
                elseif locj <= 12 % if we have more than 6 treatments, re-use colours and make broken line
                    h(j) = plot(Xall2x{i}(:,loci(j)),Xall2y{i}(:,loci(j)),[plotcol(locj-6),'--'],'LineWidth',2); % plot each model curve with a different color
                elseif locj <= 18 % if we have more than 12 treatments, re-use colours and make dot-stripe line
                    h(j) = plot(Xall2x{i}(:,loci(j)),Xall2y{i}(:,loci(j)),[plotcol(locj-12),'.-'],'LineWidth',2); % plot each model curve with a different color
                else
                    disp('cannot plot more than 18 lines yet ... so modify your script')
                end
            end
        end
        
        for j = 1:length(loci) % run through all scenarios that have data for state 1
            [dummy,locj] = ismember(loci(j),locall); % to find the right colour ...
            % [dummy,locx]=ismember(X0mat(1,loci(j)),DATAx{i}(1,2:end)); % to find right data to plot
            locx = find(X0mat(1,loci(j)) == DATAx{i}(1,2:end)); % this should work if we have more columns of data in one scenario
            if bw == 1 % plot in black and white
                if locj <= 8
                    h_tmp = plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),['k',markers_dr{locj}],'MarkerFaceColor','w');
                elseif locj <= 16
                    h_tmp = plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),['k',markers_dr{locj-8}],'MarkerFaceColor','k');
                elseif locj <= 24
                    h_tmp = plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),['k',markers_dr{locj-16}],'MarkerFaceColor','y');
                end
                h(j) = h_tmp(1); % this is needed in case there are multiple columns of extra data for one scenario
            else
                if locj <= 6
                    plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),'ko','MarkerFaceColor',plotcol(locj));
                elseif locj <= 12 % if we have more than 6 treatments, re-use colours and make squares
                    plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),'ks','MarkerFaceColor',plotcol(locj-6));
                elseif locj <= 18 % if we have more than 12 treatments, re-use colours and make triangles
                    plot(DATAx{i}(2:end,1),DATAx{i}(2:end,locx+1),'k^','MarkerFaceColor',plotcol(locj-12));
                end
            end
        end
        
        % add a legend using the cell array L
        if ~isempty(h)
            legend(h,L{loci}) % create a legend TEST
        end
        
    end
end

