function calc_likregion(par,nrs,filenm)

% Calculates a likelihood-based joint confidence region. First, profile
% likelihoods for all fitted parameters are used to find the edges of the
% hyperbox that contains the true confidence region (of which the shape is
% then still unknown). Next, Latin Hypercube sampling is used to sample the
% hyperbox, and a likelihood-ratio test is used to decide which ones belong
% to the confidence region, and which are outside.
%
% For convenience, this function also calculates the confidence intervals
% for the single parameters, just like calc_proflik.m. This helps, as there
% is no need to do the profiling twice.
%
% Second input is a vector with the number of LH samples to draw, a
% selection between a detailed (1) and course (2) profiling, and an option
% to NOT do the profiling (1, instead of 0). Instead, the boundaries will
% be taken from the saved MAT file from a previous analysis. This means
% that you do not have to do the profiling step each time you want to
% calculate a confidence region.
%
% This function applies a variable stepsize algorithm. When the likelihood
% ratio changes very little (less than Lcrit_min), the stepsize is
% increased (up to a maximum, specified by Fstep_max). When the lik. ratio
% changes too much (more than Lcrit_max), the algorithm tries again with a
% smaller stepsize (also bound to a minimum: Fstep_min). Note that the
% stepsize is used as a fraction of the parameter value that is tried. To
% prevent very small stepsizes when the value goes towards zero (as can be
% the case for NECs), I also apply an *absolute* minimum stepsize
% (Fstep_abs), which is specified as a fraction of the best parameter value
% (Xhat) (unless it is zero, then algoritm takes something small!).
%
% Author : Tjalling Jager 
% Date: Aug 2015
% Web support: http://www.debtox.info/byom.html

global names ttot
% Note, the profiles require ttot and ctot to be defined as globals, which
% is done in prelim_checks.m. These parameters are required by transfer.m.
% Only the names parameter is used here for plotting and reporting.

if exist('lhsdesign','file')~=2, % only when lhsdesign exists as an m-file in the path
    error('No confidence region calculation; you need the statistics toolbox for that. You can still do a normale profile though ...')
end

nr_lhs      = nrs(1); % extract number of samples to draw
prof_detail = nrs(2); % extract option to make profiling detailed (1) or course (2)
skip_prof   = nrs(3); % extract option to skip profiling

if skip_prof == 1 && exist([filenm,'_LR.mat']) == 2
    load([filenm,'_LR']) % load the random sample from the last calc_likregion run
    disp(' ')
    disp('Calculating sample from confidence region using previously determined bounds from MAT file.')
    drawnow % plot the last plot in the plot buffer before starting the analysis
else
    skip_prof = 0;
    disp(' ')
    disp('Calculating profiles and sample from confidence region ... please be patient.')
    drawnow % plot the last plot in the plot buffer before starting the analysis
end

if isempty(names) % if this script is run without refitting, 'names' is  not defined
    names  = fieldnames(par); % extract all field names of par (global)
end % this is probably not needed anymore ...
if exist('ttot') == 0 % see if the global ttot is available
    error('Run an optimisation at least once before profiling the likelihood.')
end
% tic % turn on the stopwatch

pmat    = packunpack(1,par,0);  % transform structure into a regular matrix
% put parameters that need to be on log scale on log10 scale (transfer needs it that way)
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1));
% also modify the min max ranges, but do that on a copy
pmat2 = pmat; % do this on a copy, as transfer wants the non-log version for the bounds
pmat2(pmat2(:,5)==0,[3 4]) = log10(pmat2(pmat2(:,5)==0,[3 4]));

parshat = pmat(:,1); % this is the best-fit parameter vector
par_sel = pmat(:,2); % this is the selection vector
ind_fit = find(par_sel == 1); % indext to fitted parameters
loglikmax = -1 * transfer(parshat(par_sel==1),pmat); % use transfer to obtain max log-likelihood

% For finding the bounds of the joint hypercube region, use number of
% parameters as dfs. This is used at the bottom of this function to obtain
% a sample from the CI for prediction intervals. Making profiles here
% restricts the space that we need to sample.
chicrit = chi2inv(0.95,sum(par_sel==1));
% to find the bounds of the hypercube, use the nr of estimated parameters as df
chicrit2 = chi2inv(0.95,1);
% to find 95% intervals for the single parameters, use 1 df
% while we're profiling anyway, the 95% single-parameter interval is obtained for free!

pmat_orig = pmat;    % remember the original parameter values (we are messing with a copy)
psel_orig = par_sel; % remember the original parameter selections (we are messing with a copy)

%% Do the profiling

if skip_prof == 0
    
    boundscoll = zeros(length(ind_fit),2); % initialise the matrix to catch the bounds of the hyperbox
    fid = fopen('profiles_newopt.out','w'); % file to save all new optima as soon as the profile finds some
    % the 'w' option destroys the old contents of the file; use 'a' to append
    
    disp('  ')
    disp('Confidence interval from the profile (single parameter, 95% confidence)')
    disp('=================================================================================')
    
    for i_p = 1:length(ind_fit)
        
        parnum  = ind_fit(i_p);   % find the index for fitted parameter i_p
        Xhat    = pmat(parnum,1); % the best estimate for the parameter (2nd column is select)
        
        % =====================================================================
        %               PERFORMANCE PARAMETERS
        % These values can be changed if the performance of the likelihood
        % procedure is not adequate.
        % =====================================================================
        
        switch prof_detail
            case 1
                Fstep_min = 0.001;    % min stepsize (as fraction of value that is tried)
                Fstep_max = 30;       % max stepsize (as fraction of value that is tried)
                if Xhat == 0,         % if parameter value is zero; than parameter is likely a NEC
                    Fstep_abs = 1e-4; % This is just a low value that should be ok in most situations
                else
                    Fstep_abs = 1e-3 * abs(Xhat); % smallest stepsize (in absolute sense)
                end
                % (this is to prevent problems when the parameter value that is tried goes
                % to zero; the stepsize is a fraction of the value tried so can also become
                % very small).
                
                % criteria for change in likelihood ratio
                Lcrit_max  = 1;   % maximum change in likelihood, above this value, stepsize is decreased
                Lcrit_min  = 0.2; % minimum change in likelihood, below this value, stepsize is increased
                Lcrit_stop = chicrit+10;  % stop when likelihood ratio reaches this value
                
            case 2
                Fstep_min = 0.005;    % min stepsize (as fraction of value that is tried)
                Fstep_max = 30;       % max stepsize (as fraction of value that is tried)
                if Xhat == 0,         % if parameter value is zero; than parameter is likely a NEC
                    Fstep_abs = 1e-3; % This is just a low value that should be ok in most situations
                else
                    Fstep_abs = 1e-2 * abs(Xhat); % smallest stepsize (in absolute sense)
                end
                % (this is to prevent problems when the parameter value that is tried goes
                % to zero; the stepsize is a fraction of the value tried so can also become
                % very small).
                
                % criteria for change in likelihood ratio
                Lcrit_max  = 2; % maximum change in likelihood, above this value, stepsize is decreased
                Lcrit_min  = 0.7; % minimum change in likelihood, below this value, stepsize is increased
                Lcrit_stop = chicrit+5;  % stop when likelihood ratio reaches this value
                
        end
        
        % =====================================================================
        
        % depart from the original values (needed as we loop over all fitted parameters)
        par_sel = psel_orig;
        pmat    = pmat_orig;
        
        par_sel(parnum) = 0; % do not fit this parameter anymore, as we make profile
        pmat(:,2) = par_sel; % overwrite the selection part of pmat with the new par_sel
        
        p_index = (par_sel==1); % find the parameters that still need to be fitted
        Xhat = parshat(parnum); % best value of the profiled parameter
        
        % set options for the fitting (rougher than for finding the best fit; increases speed)
        oldopts  = optimset('fminsearch');
        options  = optimset(oldopts,'Display','off','FunValCheck','on'); % show no iterations
        options1 = optimset(options,'TolX',1e-2,'TolFun',1e-2,'MaxFunEvals',30*(length(parshat)-1)); % only do rough optimisation
        options2 = optimset(options,'TolX',1e-2,'TolFun',1e-2,'MaxFunEvals',50*(length(parshat)-1)); % only do rough optimisation
        
        parmin  = pmat2(parnum,3); % use min and max of parameter as provided in script
        parmax  = pmat2(parnum,4); % this is from pmat2, so these may be on log-scale
        
        Xcoll     = []; % initialise the vector that collects the X-values
        loglikrat = []; % initialise the vector that collects the likrats
        
        for j = 1:2, % calculate from profile down and up from the ML estimate
            if j == 1,
                % First calculating from max. lik. estimate downwards to zero ...
                logliklowest = 0;
                proffact     = -1; % going down ...
                
                figure % initialise a figure to keep track of the progress
                h1 = gca; % remember the current axis number for plotting!
                hold on
                plot(h1,Xhat,0,'ko','MarkerFaceColor','w') % plot the max lik value as a circle
                drawnow % update the plot
                
                % if the parameter was on log-scale, mention it in the axis label
                if pmat(parnum,5) == 0
                    xlabel(h1,['parameter ',names{parnum},' (log-scale)'],'FontSize',12)
                else
                    xlabel(h1,['parameter ',names{parnum}],'FontSize',12)
                end
                
                ylabel(h1,'minus 2x log-likelihood ratio','FontSize',12)
                set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
                drawnow
                
            else
                % Now starting calculations from max. lik. estimate upwards ...
                proffact = 1; % going up ...
                % remember values from the first run (downwards) and clear 'em
                XcollD     = Xcoll;
                loglikratD = loglikrat;
                clear Xcoll loglikrat;
            end
            
            p_try     = pmat; % temporary parameter vector
            Xtry      = Xhat; % initialise the parameter to try on the ML estimate
            logliktmp = 0;
            i         = 1;
            
            % first values are zero
            Xcoll(1)     = Xtry; % all tried parameter values will be stored in Xcoll
            loglikrat(1) = 0;
            flag         = 0;
            
            % initialise Fstep as fraction of parameter value
            Fstep   = Fstep_min; % start from the minimum step size
            flagmin = 0; % test: flag is used to indicate that stepsize cannot or should not be decreased
            % this flag is put here to avoid trouble when the profile is followed
            % upwards. Each step, Fstep * Xtry will be higher than Fstep_abs, which
            % leads to a rejection of the value, a decrease in stepsize and a new
            % try.
            
            while flag == 0, % as long as we have not reached a stopping criterion
                
                flag_try = 0;
                % remember the previous values
                Xtry_old      = Xtry;
                logliktmp_old = logliktmp;
                
                while flag_try == 0 && flag == 0; % as long as we haven't got a good new value
                    
                    if Xtry_old ==0, % if old tryvalue is zero, we cannot use the relative stepsize.
                        Xtry    = Xtry_old + proffact * Fstep_abs; % try a new value, based on absolute stepsize
                        Fstep   = Fstep_min; % and re-initialise the relative stepsize
                        flagmin = 1;
                    elseif Fstep * abs(Xtry_old) >= Fstep_abs, % new stepsize is larger than abs min size
                        Xtry = Xtry_old + proffact * Fstep * abs(Xtry_old); % try a new value for the parameter
                        % use the given relative stepsize, or the minimum absolute stepsize
                        
                        % now, if we are at least 1.5 times above the absolute
                        % minimum and 1.5 times above the minimum relative stepsize
                        % ... then it makes sense to allow for a decrease in
                        % stepsize. Thus, flag is set to zero.
                        if Fstep * abs(Xtry_old) > 1.5 * Fstep_abs && Fstep > 1.5 * Fstep_min,
                            flagmin = 0;
                        else
                            flagmin = 1;
                        end
                    elseif Fstep * abs(Xtry_old) < Fstep_abs,
                        Xtry    = Xtry_old + proffact * Fstep_abs; % try a new value for the parameter
                        Fstep   = Fstep_abs / abs(Xtry_old);
                        flagmin = 1;
                    end
                    
                    % dont profile to negative values or very high ones!
                    % make sure that values are within the specified parameter bounds
                    % (bounds specified in calcstart.m)
                    Xtry = max(Xtry,parmin);
                    Xtry = min(Xtry,parmax);
                    
                    p_try(parnum,1) = Xtry ; % put new profile value into the total try vector
                    p_fit = p_try(p_index,1); % these are the ones that must be estimated now
                    
                    % Now do fminsearch to get new best fit, with profile parameter fixed
                    % Do the optimisation twice: once rough, and once a little more
                    % precise. Hopefully, this will avoid local minima, but still
                    % be rapid enough!
                    parshattemp = fminsearch('transfer',p_fit,options1,p_try);
                    [parshattemp,logliknew] = fminsearch('transfer',parshattemp,options2,p_try);
                    % minimisation is done with the reduced parameter set, other parameters in par are kept
                    % to the fixed value. This means that parshat will also contain the limited set only.
                    
                    % and make the vector whole again
                    p_try(p_index,1) = parshattemp ; % replace values in total vector with values that were fitted
                    logliktmp        = -2 * (-1*logliknew-loglikmax); % likelihood ratio that follows a chi square
                    deltaL           = abs(logliktmp - logliktmp_old); % difference with previous likelihood
                    
                    % If the change in likelihood ratio is larger than we allow, or
                    % if we are at the border of the allowable parameter range, we
                    % want to decrease the stepsize. If the smalles stepsize is
                    % already used, we can stop the analysis.
                    skippit = 0;
                    
                    if deltaL > Lcrit_max,
                        if flagmin == 0, % & logliktmp < 5,
                            % Only reject try and decrease stepsize when the flag says that it is possible!
                            % Test: and only when we are in an interesting range of likelihood values!
                            Fstep   = max(Fstep / 2 , Fstep_min); % try again with half the stepsize, unless less than min step
                            skippit = 1; % no storage for this one!
                        elseif any(Xtry == [parmin parmax]),
                            flag = 1; % if already using smallest stepsize, stop!
                        end
                    elseif any(Xtry == [parmin parmax]),% also stop when max or min is reached and deltaL < Lcrit_max!
                        flag = 1; % stop!
                    end
                    
                    % If the change is small enough, or if we want to stop the analysis anyway, store the value.
                    if skippit == 0,
                        flag_try = 1;
                        i = i+1;
                        loglikrat(i) = logliktmp; % this is 2x log-likelihood ratio
                        Xcoll(i)     = Xtry; % also remember the value of c0
                        plot(h1,Xcoll(i),loglikrat(i),'k.') % plot em on the fly!
                        drawnow
                        
                        if logliktmp < logliklowest, % if we found a lower likelihood, remember it!
                            logliklowest = logliktmp;
                            p_disp       = p_try;
                            loglik_disp  = logliknew;
                            
                            % save it to the file profiles_newopt.out
                            ti = clock; % take current time
                            fprintf(fid,'%s (%02.0f:%02.0f) \n',date,ti(4),ti(5));
                            fprintf(fid,'The profiles found a better optimum: %1g (best was %1g). \n',loglik_disp,-1*loglikmax);
                            fprintf(fid,'Parameter values for new optimum \n');
                            fprintf(fid,'=================================================================================\n');
                            nfields = length(names);
                            for ii = 1:nfields,
                                if pmat(ii,5) == 0
                                    fprintf(fid,'%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{ii},10^p_disp(ii),pmat(ii,2),10^pmat_orig(ii,1));
                                else
                                    fprintf(fid,'%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{ii},p_disp(ii),pmat(ii,2),pmat_orig(ii,1));
                                end
                            end
                            fprintf(fid,'=================================================================================\n');
                            fprintf(fid,'  \n');
                            
                        end
                        
                        if logliktmp > Lcrit_stop,% stop if probability is high enough
                            flag = 1;
                        elseif deltaL < Lcrit_min, % but change in likelihood is very small
                            Fstep = min(Fstep * 2 , Fstep_max); % so double the stepsize, unless greater than max step
                        end
                    end
                end
            end
        end
        
        diary ('results.out') % collect output in the diary "results.out"
        
        if logliklowest < 0, % if the profiler found a better optimum, display it!
            fprintf('\n  The profiles found a better optimum: %1g (best was %1g). \n',loglik_disp,-1*loglikmax);
            fprintf('Parameter values for new optimum \n');
            fprintf('=================================================================================\n');
            nfields = length(names);
            for i = 1:nfields,
                if pmat(i,5) == 0
                    fprintf('%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{i},10^p_disp(i),pmat(i,2),10^pmat_orig(i,1))
                else
                    fprintf('%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{i},p_disp(i),pmat(i,2),pmat_orig(i,1))
                end
            end
            fprintf('=================================================================================\n');
            fprintf('  \n');
        end
        
        % ================================================
        % Now, derive the intercepts with the critical value for the joint conf. region
        XcollU = Xcoll;
        loglikratU = loglikrat;
        confD = [];
        confU = [];
        
        teller  = 0;
        if loglikratD(end) < chicrit, % if the end of the profile is below the criterion
            teller = teller + 1; % count it as a boundary
            confD(teller) = min(XcollD); % take boundary as lower bound of conf range!
            upsw = 0; % tell the model that we are below the criterion
            % disp('Warning: taking the lowest parameter value as lower confidence limit')
        else
            upsw = 1; % tell the model that we are above the criterion
        end
        for i = length(loglikratD):-1:1,
            if loglikratD(i) < chicrit && upsw == 1,
                upsw = 0; % set switch to zero, we are now under the chi2 criterion
                teller = teller + 1; % found a zero crossing
                confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit,'linear'); % store this value for X
            elseif loglikratD(i) > chicrit && upsw == 0, % we were below the criterion but now cross up again
                upsw = 1;
                teller = teller + 1;
                confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit,'linear'); % store this value for X
            end
        end
        
        teller  = 0;
        if loglikratU(end) < chicrit, % if the end of the profile is below the criterion
            teller = teller + 1; % count it as a boundary
            confU(teller) = max(XcollU); % take boundary as upper bound of conf range!
            upsw = 0; % tell the model that we are below the criterion
            % disp('Warning: taking the highest parameter value as upper confidence limit')
        else
            upsw = 1; % tell the model that we are above the criterion
        end
        for i = length(loglikratU):-1:1,
            if loglikratU(i) < chicrit && upsw == 1,
                upsw = 0; % set switch to zero, we are now under the chi2 criterion
                teller = teller + 1; % found a zero crossing
                confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit,'linear'); % store this value for X
            elseif loglikratU(i) > chicrit && upsw == 0, % we were below the criterion but now cross up again
                upsw = 1;
                teller = teller + 1;
                confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit,'linear'); % store this value for X
            end
        end
        
        boundscoll(i_p,:) = [min(confD) max(confU)]; % collect the extreme bounds for the confidence region
        % note that for the log-scale pars, boundscoll will stay on log scale!
        
        % ================================================
        % Now, derive the intercepts with the critical value for the single-parameter conf. interval
        teller  = 0;
        confD = [];
        confU = [];
        if loglikratD(end) < chicrit2, % if the end of the profile is below the criterion
            teller = teller + 1; % count it as a boundary
            confD(teller) = min(XcollD); % take boundary as lower bound of conf range!
            upsw = 0; % tell the model that we are below the criterion
            disp('Warning: taking the lowest parameter value as lower confidence limit')
        else
            upsw = 1; % tell the model that we are above the criterion
        end
        for i = length(loglikratD):-1:1,
            if loglikratD(i) < chicrit2 && upsw == 1,
                upsw = 0; % set switch to zero, we are now under the chi2 criterion
                teller = teller + 1; % found a zero crossing
                confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit2,'linear'); % store this value for X
            elseif loglikratD(i) > chicrit2 && upsw == 0, % we were below the criterion but now cross up again
                upsw = 1;
                teller = teller + 1;
                confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit2,'linear'); % store this value for X
            end
        end
        
        teller  = 0;
        if loglikratU(end) < chicrit2, % if the end of the profile is below the criterion
            teller = teller + 1; % count it as a boundary
            confU(teller) = max(XcollU); % take boundary as upper bound of conf range!
            upsw = 0; % tell the model that we are below the criterion
            disp('Warning: taking the highest parameter value as upper confidence limit')
        else
            upsw = 1; % tell the model that we are above the criterion
        end
        for i = length(loglikratU):-1:1,
            if loglikratU(i) < chicrit2 && upsw == 1,
                upsw = 0; % set switch to zero, we are now under the chi2 criterion
                teller = teller + 1; % found a zero crossing
                confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit2,'linear'); % store this value for X
            elseif loglikratU(i) > chicrit2 && upsw == 0, % we were below the criterion but now cross up again
                upsw = 1;
                teller = teller + 1;
                confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit2,'linear'); % store this value for X
            end
        end
        
        % if the parameter was on log-scale, back-convert the CI
        if pmat(parnum,5) == 0
            confD = 10.^(confD);
            confU = 10.^(confU);
        end
        
        if length(confD) == 1 && length(confU) == 1,
            % no problems, it is a single interval
            fprintf('%-10s interval: %10.4g - %1.4g \n',names{parnum},confD,confU)
        else
            disp('The confidence interval is a broken set (check likelihood profile to check these figures)')
            fprintf('%-10s \n',names{parnum})
            if length(confD)>1,
                for i = 1:(length(confD)-1)/2,
                    fprintf('   interval: %10.4g - %1.4g \n',confD(i*2-1),confD(i*2))
                end
            end
            fprintf('   interval: %10.4g - %1.4g \n',confD(end),confU(end))
            if length(confU)>1,
                for i = 1:(length(confU)-1)/2,
                    fprintf('   interval: %10.4g - %1.4g \n',confU(i*2),confU(i*2-1))
                end
            end
        end
        diary off    % close results.out
        
        % And make a nice plot
        plot(h1,XcollD,loglikratD,'k-')
        plot(h1,XcollU,loglikratU,'k-')
        
        % Plot the chi-square criterion for 95% and 1 degree of freedom
        plot(h1,[min(XcollD) max(XcollU)],[chicrit chicrit],'k:')
        plot(h1,[min(XcollD) max(XcollU)],[chicrit2 chicrit2],'k--')
        drawnow
        
        % set the y-axis to allow to read the graph when lik-ratio becomes very large
        plotmin = min([loglikratD loglikratU]);
        plotmax = max(10,min(30,max([loglikratD loglikratU]))); % force plotmax between 10 and 30
        ylim(h1,[plotmin plotmax])
        
    end
    
    disp('  ')
    disp('Edges of the joint 95% confidence region (using df=p)')
    disp('=================================================================================')
    
    for i_p = 1:length(ind_fit)
        parnum  = ind_fit(i_p);   % find the index for fitted parameter i_p
        if pmat(parnum,5) == 0 % parameter is fitted on log-scale
            fprintf('%-10s interval: %10.4g - %1.4g \n',names{parnum},10^boundscoll(i_p,1),10^boundscoll(i_p,2))
        else
            fprintf('%-10s interval: %10.4g - %1.4g \n',names{parnum},boundscoll(i_p,1),boundscoll(i_p,2))
        end
    end
    % make the bounds 1% larger, just for safety
    boundscoll(:,1) = boundscoll(:,1)-0.01*abs(boundscoll(:,1));
    boundscoll(:,2) = boundscoll(:,2)+0.01*abs(boundscoll(:,1));
    
    fclose(fid); % close the file profiles_newopt.out for writing
    drawnow % plot the last plot in the plot buffer before doing the next analysis

end % this ends the skip_prof jump ...

%% Take an LHS sample (needs statistics toolbox) to find the joint conf. region
% pmat may contain parameters that are on log-scale, and boundscoll too.
% That is correct: transfer will put it back on normal scale.

% depart from the original values
par_sel = psel_orig;
pmat    = pmat_orig; % but parameter values are on log scale

sample_lhs = lhsdesign(nr_lhs,length(ind_fit)); % Latin-hypercube sample between 0 and 1
for i = 1:length(ind_fit) % go through the fitted parameters
    sample_lhs(:,i) = sample_lhs(:,i)*(boundscoll(i,2) - boundscoll(i,1))+boundscoll(i,1);
    % and change them to cover the bounds of the hypercube
end
loglik_lhs = zeros(size(sample_lhs,1),1); % pre-define for speed
for i = 1:size(sample_lhs,1) % run through all samples
    loglik_lhs(i) = -1 * transfer(sample_lhs(i,:),pmat); % use transfer to obtain likelihood for each set
end

chi_lhs = 2*(loglikmax - loglik_lhs); % calculate difference with the best fitting paramaters that follows a chi-square
ind_confreg = (chi_lhs<=chicrit); % this is the index to the sets that are within the confidence region
sample_lhs_acc = sample_lhs(ind_confreg,:); % only the accepted parameter values (within the conf. region)
% log-scale parameters are still on log-scale in this sample!

disp(' ')
disp(['Number of latin-hypercube samples taken: ',num2str(nr_lhs),', of which accepted in the conf. region: ',num2str(size(sample_lhs_acc,1))])

% save the sample in a MAT-file with the name of the script, with
% _LR at the end. Load can be used to retrieve it.
% also, save par (total parameter matrix), par_sel (selection
% matrix) and bounds to make plots without redoing the fit.
rnd = sample_lhs; % put the TOTAL sample in the parametername rnd for calc_conf
acc = sample_lhs_acc; % put the ACCEPTED sample in the parametername rnd for calc_conf
save([filenm,'_LR'],'rnd','acc','par','par_sel','boundscoll')

disp(' ')
disp(['Time required: ' secs2hms(toc)])
