function calc_pop(par_out,X0mat,loc,t,c,Th,trep,pop_txt)

% This function allows for a calculation of the intrinsic rate of increase
% using the Euler-Lotka equation. Both for continuous and discrete
% reproduction.
%
% par_out : fitted parameter structure
% X0mat   : matrix with scenarios and initial values
% loc     : vector with location of survival and repro data in the state vector
% t       : time vector for population calculations
% c       : concentration range for population calculations
% trep    : reproduction events (times; first one must be a zero)
%
% Author : Tjalling Jager
% Date   : Aug 2015
% Web support: http://www.debtox.info/byom.html

t = t(:); % make sure time is a column vector
t2 = mean([t(1:end-1) t(2:end)],2);         % new averaged time vector
rgr_min = -1;
rgr_max = 3;
plotmax = -1;
locS = loc(1); % location of survival data
locR = loc(2); % location of cumul. repro data
% I have to check whether this works well when there are multiple data sets
% for each state, but I think it will work.

if c(1) == -1 % if user wants to use only the scenarios in X0mat ...
    cpop = X0mat(1,:);
else % a scenario range (e.g., a large concentration vector) is provided
    cpop = c;
end

% if there is no need for scenarios, just define fscen with multiple
% values, and modify the just below the for loop running through the
% scenarios
fscen = [1 0.9 0.8]; % three scenarios with limited food
% fscen = 1; % dummy for when we do not want to run different scenarios (e.g., food levels)

rgr     = zeros(length(cpop),length(fscen)); % initialise rgr to catch the pop. growth rate

figure
hold on
xlabel([pop_txt{1} pop_txt{2}],'FontSize',12)
ylabel('intrinsic rate of increase (d-1)','FontSize',12)
set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting

plotcol = 'krbgcm'; % 6 colors should be enough ...

for i = 1:length(fscen)
    
    if length(fscen)>1
        % Which scenarios to run? Here, different food levels are used, which
        % means that parameter f is overwritten.
        par_out.f(1) = fscen(i); % take the next scenario, replace f in par_out
    end
    
    for j = 1:length(cpop) % run through all scenarios
        if c(1) == -1
            X0mat_tmp = X0mat(:,j); % take exact scenario for THIS concentrations!
        else
            X0mat_tmp = X0mat(:,1); % take first scenario for ALL concentrations!
            X0mat_tmp(1) = c(j); % only replace the concentration
        end
        if exist('DATAx') && ~isempty(DATAx) % extra uni-variate dataset?
            [Xout,TE,Xout2] = call_deri(t,par_out,X0mat_tmp); % use call_deri.m to provide the output for one scenario
        else % this is needed to ensure backwards compatability
            [Xout,TE] = call_deri(t,par_out,X0mat_tmp); % use call_deri.m to provide the output for one scenario
        end
        cR = Xout(:,locR);
        if locS == -1
            S = ones(size(cR)); % just assume no deaths
        else
            S = Xout(:,locS);
        end
        
        if trep(end) == 0 % continuous repro
            Rp = diff(cR)./(diff(t)); % estimated repro rate, as difference (on t2 scale)
            Sp = mean([S(1:end-1) S(2:end)],2); % survival on new averaged time vector t2
            if any(Rp>0),
                rgr(j,i) = fzero(@optpop,[rgr_min rgr_max],[],Sp,Rp,t2,Th);
            else
                rgr(j,i) = NaN;
            end
        else % repro events CHECK CHECK!
            if cR(end)>0,
                rgr(j,i) = fzero(@optpop_dis,[rgr_min rgr_max],[],S,cR,t,Th,trep);
            else
                rgr(j,i) = NaN;
            end
        end
        
        if c(1) == -1
            plot(cpop(j),rgr(j,i),'ko')
        else
            plot(cpop(j),rgr(j,i),'k.')
        end
        drawnow
    end
    
    if c(1) == -1
        plot(cpop,rgr(:,i),[plotcol(i),':'],'LineWidth',1);
    else
        hf(i) = plot(cpop,rgr(:,i),[plotcol(i),'-'],'LineWidth',1.5);
        legtxt{i} = ['f = ',num2str(fscen(i))];
    end
    plotmax = max(plotmax,max(rgr(:,i)));
    
end

ylim([0 plotmax*1.05])
if c(1) ~= -1 && length(fscen)>1
    legend(hf,legtxt)
end

figure
hold on
xlabel([pop_txt{1} pop_txt{2}],'FontSize',12)
ylabel('relative rate of increase (d-1)','FontSize',12)
set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
for i = 1:length(fscen)
    if c(1) == -1
        plot(cpop,rgr(:,i)/rgr(1,i),[plotcol(i),':'],'LineWidth',1);
    else
        hf(i) = plot(cpop,rgr(:,i)/rgr(1,i),[plotcol(i),'-'],'LineWidth',1.5);
    end
end
ylim([0 1.05])
if c(1) ~= -1 && length(fscen)>1
    legend(hf,legtxt)
end

% ===========================================================================
%      Local function! optpop
% ===========================================================================
% A tiny function to calculate the population growth rate for continuous
% reproduction
function crit = optpop(rgr,S,R,T,Th)

crit = trapz(T,R .* S .* exp(-rgr*(T+Th))) -1; % trapezium rule integration

% ===========================================================================
%      Local function! optpop_dis
% ===========================================================================
% A tiny function to calculate the population growth rate for reproduction
% in clutches (specified in trep)
function crit = optpop_dis(rgr,S,cR,t,Th,trep)

crit = -1;
for i = 1:length(trep)-1
    Re   = interp1(t,cR,trep(i+1))-interp1(t,cR,trep(i)); % cumul. repro at spawning event minus previous
    Se   = interp1(t,S,trep(i+1)); % matching survival
    crit = crit + Re * Se * exp(-rgr*(trep(i+1)+Th));
end
