% Create data sets for testing GUTS.
%
% Data sets assuming SD, IT or mixed death mechanisms. Optional real
% internal concentrations using a bioconcentration factor (PVd).
% IT and mixed with log-normal or log-logistic distribution of thresholds.
% This script requires the statistics toolbox.
%
% At this moment, all stochasticity is in the death mechanism, but it is
% easy to add stochasticity in TK as well. Damage is not used here.

% Author : Tjalling Jager (www.debtox.info)
% Version: Mar 2015

clear all
format short g

disp(' ')
disp('===============================================================')
disp('Starting simulations')
set(0,'DefaultFigureWindowStyle','docked') ; 
% use this option to collect all figure into one window with tab controls

% =========================================================================
% choose parameter values
ke  = 1;      % (dominant) elimination rate constant
z   = 0.60;   % median of NEC distribution
kk  = 0.39;   % killing rate (set high for IT)
h0  = 0.0035; % background hazard rate
PVd = 1;      % bioconcentration factor
Fs  = 2;      % factor spread of NEC distribution (set 1 for SD)

% the different scenarios, with no. animals, no. of observations, no. of
% concentrations (excl. control)
datakey = [1 20 4 8
    2 100 4 8
    3 20 10 8
    4 100 10 8
    5 20 4 5
    6 100 4 5
    7 20 10 5
    8 100 10 5];

dataset = 1;  % which scenario to take from the table (if needed, this can
% be a for loop to automatically run through the entire table)

% Take the correct values from the table
N = datakey(dataset,2); % number of test animals
T = datakey(dataset,3); % number of observation times
C = datakey(dataset,4); % number of exposure concentrations (excl. control)

casenm = 'GP_MAL'; % name of the case study (here: Gammarus pulex and malathion)
if C == 8 % for the long concentration vector
    cd   = [0 0.35 0.54 0.74 1.1 1.9 2.8 3.8];  % external concentrations
elseif C == 5 % for the short one
    cd   = [0 0.35 0.74 1.9 3.8];  % external concentrations: less in same range
end
t    = [0:T]'; % observation times (daily)
n    = N;      % number of simulated individuals

n2   = 500; % many individuals for underlying model curves (0 to skip)
dis  = 1;   % 1) log-normal, 2) log-logistic (without statistics toolbox)
nDAT = 3;   % number of data sets to produce

% =========================================================================
% save a text file with the parameter values
fnm = [casenm,'_',num2str(N),'_',num2str(T),'_',num2str(C),'_'];    
fnmpars = [fnm,'pars.txt'];
fid = fopen(fnmpars,'w');
fprintf(fid,'Parameter values: \n');
fprintf(fid,'ke = %g\n',ke);
fprintf(fid,'z  = %g\n',z);
fprintf(fid,'kk = %g\n',kk);
fprintf(fid,'h0 = %g\n',h0);
fprintf(fid,'Fs = %g\n',Fs);

% additionally, calculate mean and sd of lognormal distribution
if dis == 1
    sig_z  = log(Fs)/1.96;  % sd on ln scale, calculated from Fs
    var_z = sig_z^2; % variance on ln scale
    mu_z  = log(z); % mean threshold on ln scale
    vari = (exp(var_z)-1)*exp(2*mu_z+var_z); % variance of lognormal distribution
    sd = sqrt(vari); % standard deviation of lognormal
    mn = exp(mu_z+var_z/2); % mean of lognormal
end
fprintf(fid,'Mean and s.d. of the lognormal distribution from Fs and z: \n');
fprintf(fid,'sd = %g\n',sd);
fprintf(fid,'mn = %g\n',mn);
fclose(fid);

% =========================================================================
% create the synthetic data

X0 = [0;0]; % initial values body residues and integrated hazard rate
options = []; % options for the ODE solver
options = odeset(options, 'RelTol',1e-4,'AbsTol',1e-7); % specify tightened tolerances
options = odeset(options,'InitialStep',max(t)/1000,'MaxStep',max(t)/100); % stepsize adaptation

Fs = max(Fs,1+1e-10); % make sure Fs is slightly larger than 1
sd_NEC = log10(Fs)/1.96;  % sd on 10log scale, calculated from Fs
beta   = log(39)/log(Fs); % shape parameter for log-logistic from Fs

% mean_nec_dist = 10^(log10(z)+(sd_NEC^2)/2); % this should be the mean NEC from the lognormal

for ndat = 1:nDAT % run through replicate data sets
    
    S_coll  = zeros(length(t),length(cd));
    cV_coll = zeros(length(t),length(cd));
    z_coll = [];
    
    for j = 1:length(cd) % run through concentrations
        
        if dis == 1 % lognormal distribution
            z_range = 10.^(normrnd(log10(z),sd_NEC,n,1)); % samples for the NEC
            z_coll = [z_coll;z_range]; % collect sampled values
        else
            % log-logistic distribution with statistics toolbox
            % z_range = random('loglogistic',log(z),1/beta,n,1); % samples for the NEC
            % log-logistic without toolboxes!!
            z_range = z*((1./rand(n,1))-1).^(-1/beta); % samples for the NEC
            
        end 
        
        coll_surv = zeros(length(t),n); % collect the simulated individuals in one matrix
        coll_body = zeros(length(t),n); % also collect the body residues of each individual
        
        for i = 1:n % run through individuals
            
            par = [ke;z_range(i);kk;h0;PVd]; % give them their own parameter set
            % feel free to change other parameters per individual here too
            % par = [normrnd(ke,0.1*ke);z_range(i);kk;h0;PVd]; % e.g., normal noise on ke with CV=10%
            
            [tn1, Xout] = ode15s(@deri, t, X0, options, cd(j), par); % calculate survival from ODEs
            
            cV_out = Xout(:,1);   % body residues
            H_out  = Xout(:,2);   % integrated hazard over time
            S_out  = exp(-H_out); % survival probabilities from hazard rates
            
            p = [-diff(S_out);1+sum(diff(S_out))]; % multinomial probabilities
            p = max(0,p); % sometimes it gets slightly negative ...
            
            deaths = mnrnd(1,p); % take a sample from multinomial distribution
            S_sim = [1 1-cumsum(deaths)]; % and translate to survival data
            S_sim(end) = []; % last interval is after end of test, so remove
            
            coll_surv(:,i) = S_sim; % store the sample for each individual
            coll_body(:,i) = cV_out; % store the body residue as well
        end
        
        S_coll(:,j)  = sum(coll_surv,2); % sum individuals to survivors in cohort
        cV_coll(:,j) = mean(cV_out,2); % also store the mean body residues
        
    end
    
    disp(['Simulated data set ',num2str(ndat),' of ',num2str(nDAT)])
    DATA{ndat} = [-1 cd;t S_coll]; % data set, collected into a structure
    disp(DATA{ndat}) % and displayed on screen
        
    % =====================================================================
    % write dataset to file, subsequent replicate sets are numbered A, B, etc.
    fid = fopen([fnm,char(64+ndat),'.txt'],'wt');  
    wrtstr = '%g\t';
    
    % in format helpful for R (tab delimited, time in columns)
    for iwrt = 1:length(t)-1
        wrtstr = [wrtstr,'%g\t'];
    end
    wrtstr = [wrtstr,'%g\n'];
    fprintf(fid,wrtstr,DATA{ndat});
    fclose(fid);

%     % in format helpful for matlab (tab delimited, time in rows)
%     for iwrt = 1:length(cd)-1
%         wrtstr = [wrtstr,'%g\t'];
%     end
%     wrtstr = [wrtstr,'%g\n'];
%     fprintf(fid,wrtstr,(DATA{ndat})');
%     fclose(fid);
    
% a = load([fnm,char(64+ndat),'.txt']); % this way, the saved matrix can be read into a matrix again

% =====================================================================
    
    % plot the simulated data for visual inspection
    h1{ndat} = figure;
    subplot(1,2,1)
    hold on
    plot(tn1,cV_coll,'ko','LineWidth',1,'MarkerFaceColor','y')
    xlabel('time (days)','FontSize',12)
    ylabel('body residue','FontSize',12)
    set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
    
    subplot(1,2,2)
    hold on
    plot(tn1,S_coll,'ko:','LineWidth',1,'MarkerFaceColor','y')
    xlabel('time (days)','FontSize',12)
    ylabel('survivors','FontSize',12)
    ylim([0 n])
    set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
    
    % % You can calculate the LC50 using trimmed Spearman-Karber
    % figure
    % hold on
    % [mu,gsd,left,right] = TSK(cd,n-S_coll(end,:),n*ones(size(cd)),0.025,0.95);
    %
    % disp('===============================================================')
    % fprintf('Trimmed Spearman-Karber LC50: %g (%g - %g) \n',mu,left,right)
    % disp('===============================================================')
    % plot(cd,S_coll(end,:)/(S_coll(end,1)),'ko')
    
end
disp('===============================================================')

% % uncomment for a normal-probability plot of the thresholds for inspection
% figure
% hold on
% z_coll = log10(sort(z_coll));
% normplot(z_coll)

% calculate mean and sd of sample to compare to expected values
mean(z_coll)
std(z_coll)

% ====================================
% Now for the model lines

disp(' ')
% return % uncomment to NOT plot model lines (faster!)

drawnow

if n2 > 0
    disp('Calculating the underlying model for plotting line ...')
    t = linspace(0,max(t),100); % many time points
    
    S_coll2  = zeros(length(t),length(cd));
    cV_coll2 = zeros(length(t),length(cd));
    
    if dis == 1 % log-normal distribution
        logz_range = linspace(log10(z)-3*sd_NEC,log10(z)+3*sd_NEC,n2); % regular range of NECs
        prob_range   = normpdf(logz_range,log10(z),sd_NEC); % with their prob densities
        z_range    = 10.^logz_range; % NECs on normal scale
    else % log-logistic
        Fs2 = 999^(1/beta); % This is the fraction spread for 99.9% of the distribution
        z_range = linspace(z/(1.5*Fs2),z*Fs2,n2); % range of NECs
        prob_range = ((beta/z)*(z_range/z).^(beta-1)) ./ ((1+(z_range/z).^beta).^2); % pdf for the log-logistic (Wikipedia)
    end
    prob_range   = prob_range / sum(prob_range); % normalise the densities to exactly one
    
    for j = 1:length(cd)
        
        coll_surv2 = zeros(length(t),n2);
        coll_body2 = zeros(length(t),n2);
        
        for i = 1:n2
            
            par = [ke;z_range(i);kk;h0;PVd];
            [tn2, Xout] = ode15s(@deri, t, X0, options, cd(j), par);
            
            cV_out = Xout(:,1); % body residues
            H_out  = Xout(:,2); % integrated hazard rate
            S_out  = exp(-H_out); % survival probabilities
            
            coll_surv2(:,i) = S_out*prob_range(i); % survival prob times the prob of this NEC
            coll_body2(:,i) = cV_out; % store the body residue as well
        end
        
        S_coll2(:,j) = sum(coll_surv2,2); % sum over all NECs in the range
        cV_coll2(:,j) = mean(coll_body2,2); % mean body residue
    end
        
end

for ndat = 1:nDAT % run through data sets to plot model line in there

    set(0,'CurrentFigure',h1{ndat})
    subplot(1,2,1)
    hold on
    plot(tn2,cV_coll2,'b-','LineWidth',2)
    plot(tn1,cV_coll,'ko','LineWidth',1,'MarkerFaceColor','y') % replot, so markers are on top
    
    subplot(1,2,2)
    plot(tn2,n*S_coll2,'b-','LineWidth',2)
    S_coll = DATA{ndat}(2:end,2:end);
    plot(tn1,S_coll,'ko:','LineWidth',1,'MarkerFaceColor','y')
    
end